@@ -142,8 +142,12 @@ class ELF(MetaELF):
 
         if debug_symbols:
             self.__process_debug_file(debug_symbols)
-        elif self.loader._load_debug_info and self.build_id:
-            debug_filename = '/usr/lib/debug/.build-id/%s/%s.debug' % (self.build_id[:2], self.build_id[2:])
+        elif self.loader._load_debug_info:
+            if self.build_id:
+                debug_filename = '/usr/lib/debug/.build-id/%s/%s.debug' % (self.build_id[:2], self.build_id[2:])
+                if os.path.isfile(debug_filename):
+                    self.__process_debug_file(debug_filename)
+            debug_filename = os.path.join('/usr/lib/debug', os.path.realpath(self.binary))
             if os.path.isfile(debug_filename):
                 self.__process_debug_file(debug_filename)
 
@@ -547,6 +551,9 @@ class ELF(MetaELF):
         self.extra_load_path = self.__parse_rpath(runpath, rpath)
 
         strtab = seg_readelf._get_stringtable()
+        if strtab is None:
+            l.warning("Unexpected return value from pyelftools: stringtable object is None.")
+            return
         self.__neuter_streams(strtab)
 
         # To extract symbols from binaries without section headers, we need to hack into pyelftools.
@@ -617,7 +624,9 @@ class ELF(MetaELF):
                         'sh_addralign': 0,
                     }
                     readelf_relocsec = elffile.RelocationSection(fakerelheader, 'reloc_cle', self._reader)
+                    # support multiple versions of pyelftools
                     readelf_relocsec.stream = self.memory
+                    readelf_relocsec._stream = self.memory
                     readelf_relocsec.elffile = None
                     self.__register_relocs(readelf_relocsec, dynsym)
 
@@ -636,7 +645,9 @@ class ELF(MetaELF):
                         'sh_addralign': 0,
                     }
                     readelf_jmprelsec = elffile.RelocationSection(fakejmprelheader, 'jmprel_cle', self._reader)
+                    # support multiple versions of pyelftools
                     readelf_jmprelsec.stream = self.memory
+                    readelf_jmprelsec._stream = self.memory
                     readelf_jmprelsec.elffile = None
                     self.__register_relocs(readelf_jmprelsec, dynsym, force_jmprel=True)
 
@@ -738,10 +749,11 @@ class ELF(MetaELF):
                 self.jmprel[reloc.symbol.name] = reloc
 
     def __register_tls(self, seg_readelf):
-        self.tls_used = True
         self.tls_block_size = seg_readelf.header.p_memsz
         self.tls_data_size = seg_readelf.header.p_filesz
         self.tls_data_start = AT.from_lva(seg_readelf.header.p_vaddr, self).to_rva()
+        if self.tls_block_size != 0 or self.tls_data_size != 0:
+            self.tls_used = True
 
     def __register_relro(self, segment_relro):
         segment_relro = ELFSegment(segment_relro, relro=True)
