@@ -1,15 +1,18 @@
 # -*-coding:utf8 -*-
 # This file is part of Mach-O Loader for CLE.
 # Contributed December 2016 by Fraunhofer SIT (https://www.sit.fraunhofer.de/en/).
-
+from collections import defaultdict
 from os import SEEK_CUR, SEEK_SET
 import struct
 import sys
-from io import BytesIO
+from io import BytesIO, BufferedReader
+from typing import Optional, DefaultDict, List, Tuple
+
+
 import archinfo
 
 from .section import MachOSection
-from .symbol import SymbolTableSymbol
+from .symbol import SymbolTableSymbol, AbstractMachOSymbol
 from .segment import MachOSegment
 from .binding import BindingHelper, read_uleb
 from .. import Backend, register_backend
@@ -20,6 +23,30 @@ l = logging.getLogger(name=__name__)
 
 __all__ = ('MachO', 'MachOSection', 'MachOSegment')
 
+from sortedcontainers import SortedKeyList
+
+# pylint: disable=abstract-method
+class SymbolList(SortedKeyList):
+    _symbol_cache: DefaultDict[Tuple[str, int],
+                               List[AbstractMachOSymbol]]
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self._symbol_cache = defaultdict(list)
+
+    def add(self, value: AbstractMachOSymbol):
+        super().add(value)
+        self._symbol_cache[(value.name, value.library_ordinal,)].append(value)
+
+    def get_by_name_and_ordinal(self, name: str, ordinal: int, include_stab=False) -> List[AbstractMachOSymbol]:
+        if include_stab:
+            return self._symbol_cache[(name, ordinal)]
+        else:
+            return [symbol for symbol in self._symbol_cache[(name, ordinal)] if not symbol.is_stab]
+
+# pylint: enable =abstract-method
+
+
 
 class MachO(Backend):
     """
@@ -44,6 +71,7 @@ class MachO(Backend):
         l.warning('The Mach-O backend is not well-supported. Good luck!')
 
         super().__init__(*args, **kwargs)
+        self.symbols = SymbolList(key=self._get_symbol_relative_addr)
 
         self.struct_byteorder = None  # holds byteorder for struct.unpack(...)
         self._mapped_base = None # temporary holder für mapped base derived via loading
@@ -65,9 +93,9 @@ class MachO(Backend):
         self.mod_init_func_pointers = []  # may be TUMB interworking
         self.mod_term_func_pointers = []  # may be THUMB interworking
         self.export_blob = None  # exports trie
-        self.binding_blob = None  # binding information
-        self.lazy_binding_blob = None  # lazy binding information
-        self.weak_binding_blob = None  # weak binidng information
+        self.binding_blob: Optional[bytes] = None  # binding information
+        self.lazy_binding_blob: Optional[bytes] = None  # lazy binding information
+        self.weak_binding_blob: Optional[bytes] = None  # weak binidng information
         self.symtab_offset = None # offset to the symtab
         self.symtab_nsyms = None # number of symbols in the symtab
         self.binding_done = False # if true binding was already done and do_bind will be a no-op
@@ -158,7 +186,12 @@ class MachO(Backend):
         self._parse_exports()
         self._parse_symbols(binary_file)
         self._parse_mod_funcs()
-        self.mapped_base = self._mapped_base
+
+        text_segment = self.find_segment_by_name("__TEXT")
+        if not text_segment is None:
+            self.mapped_base = text_segment.vaddr
+        else:
+            l.warning("No text segment found")
 
 
     @staticmethod
@@ -216,7 +249,6 @@ class MachO(Backend):
 
     def _resolve_entry(self):
         if self.entryoff:
-            self._mapped_base = self.find_segment_by_name("__TEXT").vaddr
             self._entry = self.entryoff
         elif self.unixthread_pc:
             self._entry = self.unixthread_pc
@@ -225,7 +257,7 @@ class MachO(Backend):
             self._entry = 0
 
     @staticmethod
-    def _read(fp, offset, size):
+    def _read(fp: BufferedReader, offset: int, size: int) -> bytes:
         """
         Simple read abstraction, reads size bytes from offset in file
         :param offset: Offset to seek() to
@@ -508,13 +540,13 @@ class MachO(Backend):
         l.debug("Adding library %r", lib_name)
         self.imported_libraries.append(lib_name)
 
-    def _load_dyld_info(self, f, offset):
+    def _load_dyld_info(self, f: BufferedReader, offset):
         """
         Extracts information blobs for rebasing, binding and export
         """
         (_, _, roff, rsize, boff, bsize, wboff, wbsize, lboff, lbsize, eoff, esize) = self._unpack("12I", f, offset, 48)
 
-        def blob_or_None(f,off,size): # helper
+        def blob_or_None(f: BufferedReader, off: int, size: int ) -> Optional[bytes]:  # helper
             return self._read(f,off,size) if off != 0 and size != 0 else None
 
         # Extract data blobs
@@ -702,13 +734,11 @@ class MachO(Backend):
 
         return result
 
-    def get_symbol_by_insertion_order(self, idx):
+    def get_symbol_by_insertion_order(self, idx: int) -> AbstractMachOSymbol:
         """
 
         :param idx: idx when this symbol was inserted
-        :type idx: int
         :return:
-        :rtype: AbstractMachOSymbol
         """
         return self._ordered_symbols[idx]
 
