@@ -5,9 +5,9 @@ class PESection(Section):
     Represents a section for the PE format.
     """
     def __init__(self, pe_section, remap_offset=0):
-        super(PESection, self).__init__(
+        super().__init__(
             pe_section.Name.decode(),
-            pe_section.Misc_PhysicalAddress,
+            pe_section.PointerToRawData,
             pe_section.VirtualAddress + remap_offset,
             pe_section.Misc_VirtualSize,
         )
