@@ -1,7 +1,8 @@
 import logging
 
 from . import Backend
-from .symbol import Symbol
+from ..errors import CLEInvalidBinaryError
+from .symbol import Symbol, SymbolType
 from ..address_translator import AT
 
 l = logging.getLogger(name=__name__)
@@ -30,51 +31,59 @@ class Relocation:
         if self.symbol is not None and self.symbol.is_import:
             self.owner.imports[self.symbol.name] = self
 
-    def resolve_symbol(self, solist, bypass_compatibility=False, thumb=False): # pylint: disable=unused-argument
+    AUTO_HANDLE_NONE = False
+
+    def resolve_symbol(self, solist, thumb=False, extern_object=None, **kwargs): # pylint: disable=unused-argument
         if self.resolved:
-            return True
+            return
+
+        if (self.symbol is None or self.symbol.type == SymbolType.TYPE_NONE) and self.AUTO_HANDLE_NONE:
+            self.resolve(None, extern_object=extern_object)
+            return
 
         if self.symbol.is_static or self.symbol.is_local:
             # A static or local symbol should only be resolved by itself.
-            self.resolve(self.symbol)
-            return True
+            self.resolve(self.symbol, extern_object=extern_object)
+            return
 
         weak_result = None
         for so in solist:
             symbol = so.get_symbol(self.symbol.name)
             if symbol is not None and symbol.is_export:
                 if not symbol.is_weak:
-                    self.resolve(symbol)
-                    return True
+                    self.resolve(symbol, extern_object=extern_object)
+                    return
                 elif weak_result is None:
                     weak_result = symbol
             # TODO: Was this check obsolted by the addition of is_static?
             # I think right now symbol.is_import = !symbol.is_export
             elif symbol is not None and not symbol.is_import and so is self.owner:
                 if not symbol.is_weak:
-                    self.resolve(symbol)
-                    return True
+                    self.resolve(symbol, extern_object=extern_object)
+                    return
                 elif weak_result is None:
                     weak_result = symbol
 
         if weak_result is not None:
-            self.resolve(weak_result)
-            return True
+            self.resolve(weak_result, extern_object=extern_object)
+            return
 
         if self.symbol.is_weak:
-            return False
+            return
 
-        new_symbol = self.owner.loader.extern_object.make_extern(self.symbol.name, sym_type=self.symbol._type, thumb=thumb)
-        self.resolve(new_symbol)
-        return True
+        new_symbol = extern_object.make_extern(self.symbol.name, sym_type=self.symbol._type, thumb=thumb)
+        self.resolve(new_symbol, extern_object=extern_object)
 
-    def resolve(self, obj):
+    def resolve(self, obj, **kwargs):  # pylint: disable=unused-argument
         self.resolvedby = obj
         self.resolved = True
         if self.symbol is not None:
             if obj is not None:
                 l.debug('%s from %s resolved by %s from %s at %#x', self.symbol.name, self.owner.provides, obj.name, obj.owner.provides, obj.rebased_addr)
             self.symbol.resolve(obj)
+        else:
+            if not self.AUTO_HANDLE_NONE:
+                raise TypeError("Programming error: resolving symbol with None but does not have CAN_HANDLE_NONE set")
 
     @property
     def rebased_addr(self):
@@ -96,16 +105,14 @@ class Relocation:
         l.error('Value property of Relocation must be overridden by subclass!')
         return 0
 
-    def relocate(self, solist, bypass_compatibility=False):
+    def relocate(self):
         """
         Applies this relocation. Will make changes to the memory object of the
         object it came from.
 
         This implementation is a generic version that can be overridden in subclasses.
-
-        :param solist:       A list of objects from which to resolve symbols.
         """
-        if not self.resolve_symbol(solist, bypass_compatibility):
+        if not self.resolved:
             return False
 
         self.owner.memory.pack_word(self.dest_addr, self.value)
