@@ -62,9 +62,9 @@ class Symbol:
 
     def __repr__(self):
         if self.is_import:
-            return '<Symbol "%s" in %s (import)>' % (self.name, self.owner.provides)
+            return '<Symbol "%s" in %s (import)>' % (self.name, self.owner.binary_basename)
         else:
-            return '<Symbol "%s" in %s at %#x>' % (self.name, self.owner.provides, self.rebased_addr)
+            return '<Symbol "%s" in %s at %#x>' % (self.name, self.owner.binary_basename, self.rebased_addr)
 
     def resolve(self, obj):
         self.resolved = True
@@ -129,3 +129,9 @@ class Symbol:
             Symbol._complained_owner = True
             l.critical("Deprecation warning: use symbol.owner instead of symbol.owner_obj")
         return self.owner
+
+    def __getstate__(self):
+        return dict((k, v) for k, v in self.__dict__.items() if k != 'owner')
+
+    def __setstate__(self, state):
+        self.__dict__.update(state)
