@@ -3,6 +3,7 @@ import sys
 import platform
 import logging
 from collections import OrderedDict
+from typing import Optional, List
 
 import archinfo
 from archinfo.arch_soot import ArchSoot
@@ -72,6 +73,10 @@ class Loader:
 
     More keys are defined on a per-backend basis.
     """
+    # _main_binary_path: str
+    memory: Optional['Clemory']
+    main_object: Optional['Backend']
+    tls: Optional['ThreadManager']
 
     def __init__(self, main_binary, auto_load_libs=True, concrete_target = None,
                  force_load_libs=(), skip_libs=(),
@@ -119,13 +124,13 @@ class Loader:
 
         self.aslr = aslr
         self.page_size = page_size
-        self.memory = None # type: Clemory
-        self.main_object = None # type: Backend
-        self.tls = None # type: ThreadManager
-        self._kernel_object = None # type: KernelObject
-        self._extern_object = None # type: ExternObject
+        self.memory = None
+        self.main_object = None
+        self.tls = None
+        self._kernel_object = None # type: Optional[KernelObject]
+        self._extern_object = None # type: Optional[ExternObject]
         self.shared_objects = OrderedDict()
-        self.all_objects = []  # this list should always be sorted by min_addr
+        self.all_objects = []  # type: List[Backend]
         self.requested_names = set()
         if arch is not None:
             self._main_opts.update({'arch': arch})
@@ -191,6 +196,16 @@ class Loader:
                 return obj
         return None
 
+    @property
+    def elfcore_object(self):
+        """
+        If a corefile was loaded, this returns the actual core object instead of the main binary
+        """
+        for obj in self.all_objects:
+            if isinstance(obj, ELFCore):
+                return obj
+        return None
+
     @property
     def extern_object(self):
         """
@@ -219,7 +234,7 @@ class Loader:
         return self._extern_object
 
     @property
-    def kernel_object(self):
+    def kernel_object(self) -> 'KernelObject':
         """
         Return the object used to provide addresses to syscalls.
 
@@ -709,14 +724,15 @@ class Loader:
         # produce dependency-ordered list of objects and soname map
 
         ordered_objects = []
-        soname_mapping = OrderedDict((obj.provides, obj) for obj in objects if obj.provides)
+        soname_mapping = OrderedDict((obj.provides if not self._ignore_import_version_numbers else obj.provides.rstrip('.0123456789'), obj) for obj in objects if obj.provides)
         seen = set()
         def visit(obj):
             if id(obj) in seen:
                 return
             seen.add(id(obj))
 
-            dep_objs = [soname_mapping[dep_name] for dep_name in obj.deps if dep_name in soname_mapping]
+            stripped_deps = [dep if not self._ignore_import_version_numbers else dep.rstrip('.0123456789') for dep in obj.deps]
+            dep_objs = [soname_mapping[dep_name] for dep_name in stripped_deps if dep_name in soname_mapping]
             for dep_obj in dep_objs:
                 visit(dep_obj)
 
@@ -741,7 +757,8 @@ class Loader:
             for obj in ordered_objects:
                 l.info("Linking %s", obj.binary)
                 sibling_objs = list(obj.parent_object.child_objects) if obj.parent_object is not None else []
-                dep_objs = [soname_mapping[dep_name] for dep_name in obj.deps if dep_name in soname_mapping]
+                stripped_deps = [dep if not self._ignore_import_version_numbers else dep.rstrip('.0123456789') for dep in obj.deps]
+                dep_objs = [soname_mapping[dep_name] for dep_name in stripped_deps if dep_name in soname_mapping]
                 main_objs = [self.main_object] if self.main_object is not obj else []
                 for reloc in obj.relocs:
                     reloc.resolve_symbol(main_objs + preload_objects + sibling_objs + dep_objs + [obj], extern_object=extern_obj)
@@ -836,11 +853,11 @@ class Loader:
             if close:
                 binary_stream.close()
 
-    def _map_object(self, obj):
+    def _map_object(self, obj: 'Backend'):
         """
         This will integrate the object into the global address space, but will not perform relocations.
         """
-        obj_size = obj.max_addr - obj.min_addr
+        obj_size = obj.max_addr - obj.min_addr + 1
 
         if obj.pic:
             if obj._custom_base_addr is not None and self._is_range_free(obj._custom_base_addr, obj_size):
@@ -856,7 +873,7 @@ class Loader:
 
             obj.rebase(base_addr)
         else:
-            if obj._custom_base_addr is not None and not isinstance(obj, Blob):
+            if obj._custom_base_addr is not None and obj.linked_base != obj._custom_base_addr and not isinstance(obj, Blob):
                 l.warning("%s: base_addr was specified but the object is not PIC. "
                           "specify force_rebase=True to override", obj.binary_basename)
             base_addr = obj.linked_base
@@ -866,7 +883,7 @@ class Loader:
         assert obj.mapped_base >= 0
 
         if obj.has_memory:
-            assert obj.min_addr < obj.max_addr
+            assert obj.min_addr <= obj.max_addr
             l.info("Mapping %s at %#x", obj.binary, base_addr)
             self.memory.add_backer(base_addr, obj.memory)
         obj._is_mapped = True
@@ -884,25 +901,25 @@ class Loader:
             # HACK: On small arches, we should be more aggressive in packing stuff in.
             gap_start = 0
         else:
-            gap_start = ALIGN_UP(self.main_object.max_addr, self._rebase_granularity)
+            gap_start = ALIGN_UP(self.main_object.max_addr + 1, self._rebase_granularity)
         for o in self.all_objects:
             if gap_start + size <= o.min_addr:
                 break
             else:
-                gap_start = ALIGN_UP(o.max_addr, self._rebase_granularity)
+                gap_start = ALIGN_UP(o.max_addr + 1, self._rebase_granularity)
 
-        if gap_start + size >= 2**self.main_object.arch.bits:
+        if gap_start + size > 2**self.main_object.arch.bits:
             raise CLEOperationError("Ran out of room in address space")
 
         return gap_start
 
     def _is_range_free(self, va, size):
         # self.main_object should not be None here
-        if va < 0 or va + size >= 2**self.main_object.arch.bits:
+        if va < 0 or va + size > 2**self.main_object.arch.bits:
             return False
 
         for o in self.all_objects:
-            if o.min_addr <= va < o.max_addr or va <= o.min_addr < va + size:
+            if o.min_addr <= va <= o.max_addr or va <= o.min_addr < va + size:
                 return False
 
         return True
