@@ -42,7 +42,7 @@ class ClemoryBase:
         try:
             start, backer = next(self.backers(addr))
         except StopIteration:
-            raise KeyError(addr)
+            raise KeyError(addr) # pylint: disable=raise-missing-from
 
         if start > addr:
             raise KeyError(addr)
@@ -52,7 +52,7 @@ class ClemoryBase:
         except struct.error as e:
             if len(backer) - (addr - start) >= struct.calcsize(fmt):
                 raise e
-            raise KeyError(addr)
+            raise KeyError(addr) # pylint: disable=raise-missing-from
 
     def unpack_word(self, addr, size=None, signed=False, endness=None):
         """
@@ -94,17 +94,17 @@ class ClemoryBase:
         try:
             start, backer = next(self.backers(addr))
         except StopIteration:
-            raise KeyError(addr)
+            raise KeyError(addr) # pylint: disable=raise-missing-from
 
         if start > addr:
-            raise KeyError(addr)
+            raise KeyError(addr) # pylint: disable=raise-missing-from
 
         try:
             return struct.pack_into(fmt, backer, addr - start, *data)
         except struct.error as e:
             if len(backer) - (addr - start) >= struct.calcsize(fmt):
                 raise e
-            raise KeyError(addr)
+            raise KeyError(addr) # pylint: disable=raise-missing-from
 
     def pack_word(self, addr, data, size=None, signed=False, endness=None):
         """
@@ -159,25 +159,16 @@ class Clemory(ClemoryBase):
     Accesses can be made with [index] notation.
     """
 
-    __slots__ = ('_backers', '_root', 'consecutive', 'min_addr', 'max_addr', 'concrete_target' )
+    __slots__ = ('_backers', '_root', 'consecutive', 'min_addr', 'max_addr')
 
     def __init__(self, arch, root=False):
         super().__init__(arch)
         self._backers = []  # type: List[Tuple[int, Union[bytearray, Clemory, List[int]]]]
         self._root = root
-
         self.consecutive = True
         self.min_addr = 0
         self.max_addr = 0
 
-        self.concrete_target = None
-
-    def is_concrete_target_set(self):
-        return self.concrete_target is not None
-
-    def set_concrete_target(self, concrete_target):
-        self.concrete_target = concrete_target
-
     def add_backer(self, start, data):
         """
         Adds a backer to the memory.
@@ -233,11 +224,6 @@ class Clemory(ClemoryBase):
                     yield start + x
 
     def __getitem__(self, k):
-        # concrete memory read
-        if self.is_concrete_target_set():
-            # l.debug("invoked get_byte %x" % (k))
-            return self.concrete_target.read_memory(k, 1)
-
         for start, data in self._backers:
             if type(data) in (bytearray, list):
                 if 0 <= k - start < len(data):
@@ -293,7 +279,6 @@ class Clemory(ClemoryBase):
             'consecutive': self.consecutive,
             'min_addr': self.min_addr,
             'max_addr': self.max_addr,
-            'concrete_target': self.concrete_target
         }
 
         return s
@@ -306,7 +291,6 @@ class Clemory(ClemoryBase):
         self.consecutive = s['consecutive']
         self.min_addr = s['min_addr']
         self.max_addr = s['max_addr']
-        self.concrete_target = s['concrete_target']
 
     def backers(self, addr=0):
         """
@@ -316,7 +300,7 @@ class Clemory(ClemoryBase):
         :param addr:    An optional starting address - all backers before and not including this
                         address will be skipped.
         """
-        started = addr <= 0
+        started = False
         for start, backer in self._backers:
             if not started:
                 end = start + backer.max_addr if type(backer) is Clemory else start + len(backer)
@@ -336,12 +320,6 @@ class Clemory(ClemoryBase):
         Reading will stop at the beginning of the first unallocated region found, or when
         `n` bytes have been read.
         """
-
-        # concrete memory read
-        if self.concrete_target is not None:
-            # l.debug("invoked read_bytes %x %x" % (addr, n))
-            return self.concrete_target.read_memory(addr, n)
-
         views = []
 
         for start, backer in self.backers(addr):
@@ -453,6 +431,9 @@ class Clemory(ClemoryBase):
                         is_consecutive = False
                     # Update the predicted starting address
                     next_start = start + backer.max_addr
+
+                if not backer.consecutive:
+                    is_consecutive = False
             else:
                 raise TypeError("Unsupported backer type %s." % type(backer))
 
